{Bibliothque pour DESMOL}
Unit oemath;

Interface

Uses
  SysUtils, WinTypes, WinProcs, Messages,
  Controls, StdCtrls, dialogs, math;

type atomes = record
  nom          : string[4];
  x,y,z        : real;
  beta         : array[1..6] of real;
  no_element   : integer;
end;

type typ_liaisons = record
   atome1 : string[2];
   atome2 : string[2];
   distance : real;
end;

type liai = record
   atome1 : integer;
   atome2 : integer;
end;

type elem = record
   couleur     : integer;
   rayon       : real;
   nom         : string[2];
end;

type
  matrice3                         = array[1..3,1..3] of real ;
  vecteur3                         = array[1..3] of real;
  vecteur4                         = array[1..4] of real;
  tab_reel                         = array[1..10] of real;

const
  pir         = 57.2957;

var
  {Variables communes concernant le cristal}
  ca,cb,cc,ae,be,ce,cae,cbe,cce           : real ;
  groupe_espace,systeme,bravais           : string;
  det_mat                                 : real;
  ibrav                                   : integer;

  {Noms}
  nomstruc                                : string;
  nom_fichier_shelx                       : string;
  nom_fichier_usr                         : string;
  nom_fichier_des                         : string;
  nom_fichier_opt                         : string;
  type_fichier_shelx                      : integer;

  {Maille}
  cel,celr                                : array[1..6] of real;
  a,b,c,alpha,beta,gamma                  : real;

  //Tableau des positions
  tp                                      : array[1..192,1..3,1..4]of real;
  npos,centre                             : integer;
  tx,ty,tz                                : array[1..4] of real;
  nbtrans                                 : integer;

  {Tableau des positions slectionnes}
  tp_sel                                  : array[1..192,1..3,1..4] of real;
  npos_sel                                : integer;

  //Atomes du motif asymtrique
  motif                                   : array[1..200] of atomes;
  nb_motif                                : integer;

  //Atomes gnrs
  child                                   : array[1..1000] of atomes;
  nb_child                                : integer;

  //Types d'lments
  element                                 : array[1..20] of elem;
  nb_typ_element                          : integer;

  //Type de liaisons
  tableau_liai                            : array[1..100] of typ_liaisons;
  nb_typ_liai                             : integer;

  //Liaisons tablies
  bonds                                   : array[1..5000] of liai;
  nb_bonds                                : SmallInt;

  {Matrices}
  r10,r01,r21,r12,r13,r31,r23,r32,r20,r02 : matrice3;
  r30,r03                                 : matrice3;

function uptrim(var s:string): string;
procedure add_zero(var s:string);
function  Det      (Mat:matrice3): Real;
procedure matinv   (Mat : matrice3; var mati :matrice3);
Procedure ident    (mat : matrice3; var mati :matrice3);
procedure trspo    (var mat : matrice3);
procedure trfo     (u1,u2,u3: real; var v1,v2,v3 : real; var r : matrice3);
procedure trfo2    (var u1,u2,u3 : real; var r : matrice3);
procedure trfo3    (var v : vecteur3; var r : matrice3);
procedure mat_mult (ma,mb : matrice3; var mc : matrice3);
Procedure prodvec  (x,y: vecteur3; var z :vecteur3);

procedure setmat;
procedure wr            (var nomf:TextFile; Titre:string; Mat:matrice3);
procedure norm          (var p,q,r : real);
procedure normd         (var x,y,z : real);
procedure normr         (var x,y,z : real);
procedure rot           (p0,q0,r0  : real; icas:integer);
Procedure erreur        (s:string);
procedure set_typ_liaisons;
procedure ecrit_fichier_usr;
procedure ecrit_fichier_opt;
procedure lit_fichier_usr;
procedure dec(s:string; var tableau:tab_reel; var no,ier:integer);
procedure licarte;

implementation
{R+}

procedure add_zero(var s:string);{
---------------------------------}
begin
   if(s = '')then s:='0';
   if(s[length(s)] = '.') then s:=s+'0';
end;

function  Det      (Mat:matrice3): Real;{
----------------------------------------}
begin
  det:=mat[1,1]*(mat[2,2]*mat[3,3]-mat[3,2]*mat[2,3])
      -mat[1,2]*(mat[2,1]*mat[3,3]-mat[3,1]*mat[2,3])
      +mat[1,3]*(mat[2,1]*mat[3,2]-mat[3,1]*mat[2,2]);
end;

{Inversion de matrice}
procedure matinv   (Mat : matrice3; var Mati : matrice3);{
---------------------------------------------------------}
var
  i,j      : integer;

begin
  mati[1,1]:=mat[2,2]*mat[3,3]-mat[2,3]*mat[3,2];
  mati[1,2]:=mat[1,3]*mat[3,2]-mat[1,2]*mat[3,3];
  mati[1,3]:=mat[1,2]*mat[2,3]-mat[1,3]*mat[2,2];

  mati[2,1]:=mat[2,3]*mat[3,1]-mat[2,1]*mat[3,3];
  mati[2,2]:=mat[1,1]*mat[3,3]-mat[1,3]*mat[3,1];
  mati[2,3]:=mat[1,3]*mat[2,1]-mat[1,1]*mat[2,3];

  mati[3,1]:=mat[2,1]*mat[3,2]-mat[2,2]*mat[3,1];
  mati[3,2]:=mat[1,2]*mat[3,1]-mat[1,1]*mat[3,2];
  mati[3,3]:=mat[1,1]*mat[2,2]-mat[1,2]*mat[2,1];

  det_mat:=det(Mat);
  if(abs(det_mat) < 0.00001)then begin
    erreur('Matrice singulire');
    exit;
  end;
  For i:=1 to 3 do begin
    For j:= 1 to 3 do begin
      mati[i,j]:= mati[i,j]/det_mat;
    end;
  end;
end;

Procedure ident (mat : matrice3; var mati :matrice3);{
-----------------------------------------------------}
var i,j  :integer;
begin
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
      mati[i,j]:=mat[i,j];
    end;
  end;
end;

procedure trspo    (var mat : matrice3);{
----------------------------------------}
var
  i,j  :integer;
  mati :matrice3;
begin
  ident(mat,mati);
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
      mat[i,j]:=mati[j,i];
    end;
  end;
end;

procedure trfo (u1,u2,u3: real; var v1,v2,v3 : real; var r : matrice3);{
-----------------------------------------------------------------------}
begin
  v1:=u1*r[1,1]+u2*r[1,2]+u3*r[1,3];
  v2:=u1*r[2,1]+u2*r[2,2]+u3*r[2,3];
  v3:=u1*r[3,1]+u2*r[3,2]+u3*r[3,3];
end;

procedure trfo2(var u1,u2,u3 : real; var r : matrice3);{
-------------------------------------------------------}
var
  v1,v2,v3  : real;
begin
  v1:=u1*r[1,1]+u2*r[1,2]+u3*r[1,3];
  v2:=u1*r[2,1]+u2*r[2,2]+u3*r[2,3];
  v3:=u1*r[3,1]+u2*r[3,2]+u3*r[3,3];
  u1:=v1;
  u2:=v2;
  u3:=v3
end;

procedure trfo3(var v : vecteur3; var r : matrice3);{
----------------------------------------------------}
var
  v1,v2,v3  : real;
begin
  v1:=v[1]*r[1,1]+v[2]*r[1,2]+v[3]*r[1,3];
  v2:=v[1]*r[2,1]+v[2]*r[2,2]+v[3]*r[2,3];
  v3:=v[1]*r[3,1]+v[2]*r[3,2]+v[3]*r[3,3];
  v[1]:=v1;
  v[2]:=v2;
  v[3]:=v3;
end;

{Produit vectoriel}
Procedure prodvec(x,y: vecteur3; var z :vecteur3);{
--------------------------------------------------}
begin
   z[1]:=x[2]*y[3]-x[3]*y[2];
   z[2]:=x[3]*y[1]-x[1]*y[3];
   z[3]:=x[1]*y[2]-x[2]*y[1];
end;

{Multiplication de matrices}
procedure mat_mult (ma,mb: matrice3; var mc : matrice3);{
--------------------------------------------------------}
var
  i,j,k :integer;
begin
  for k:=1 to 3 do begin
    for j:=1 to 3 do begin
      mc[j,k]:=0;
      for i:=1 to 3 do begin
        mc[j,k]:=mc[j,k]+ma[j,i]*mb[i,k];
      end;
    end;
  end;
end;

{Calcul des diffrentes matrices}
procedure setmat;{
-----------------}
var
      sa,sb,sc,{sae,}sbe,sce               :real;
      q,v                                  :real;
      i,j                                  :integer;
      aux                                  :matrice3;
begin
      a:=cel[1];
      b:=cel[2];
      c:=cel[3];
      alpha:=cel[4];
      beta:=cel[5];
      gamma:=cel[6];
      ca:=cos(cel[4]/pir);
      cb:=cos(cel[5]/pir);
      cc:=cos(cel[6]/pir);
      sa:=sin(cel[4]/pir);
      sb:=sin(cel[5]/pir);
      sc:=sin(cel[6]/pir);
      q:=1.-ca*ca-cb*cb-cc*cc+2.*ca*cb*cc;
      v:=a*b*c*sqrt(q);
      celr[1]:=b*c*sa/v;
      celr[2]:=c*a*sb/v;
      celr[3]:=a*b*sc/v;
      cae:=(cb*cc-ca)/(sb*sc);
      cbe:=(cc*ca-cb)/(sc*sa);
      cce:=(ca*cb-cc)/(sa*sb);
      celr[4]:=arccos(cae);
      celr[4]:=celr[4]*pir;
      celr[5]:=arccos(cbe);
      sbe:=sin(celr[5]);
      celr[5]:=celr[5]*pir;
      celr[6]:=arccos(cce);
      sce:=sin(celr[6]);
      celr[6]:=celr[6]*pir;
      ae:=celr[1];
      be:=celr[2];
      ce:=celr[3];

{-----Calcul de la matrice de passage du r.r. au res. rl et inversemt}
      r01[1,1]:=ae*ae;
      r01[1,2]:=ae*be*cce;
      r01[1,3]:=ae*ce*cbe;
      r01[2,2]:=be*be;
      r01[2,3]:=ce*be*cae;
      r01[3,3]:=ce*ce;

      r10[1,1]:=a*a;
      r10[1,2]:=a*b*cc;
      r10[1,3]:=a*c*cb;
      r10[2,2]:=b*b;
      r10[2,3]:=c*b*ca;
      r10[3,3]:=c*c;

      for i:=1 to 3 do begin
        for j:=1 to 3 do begin
          r01[j,i]:=r01[i,j];
          r10[j,i]:=r10[i,j];
        end;
      end;

{-----Orthonormalisation du rseau rciproque R21}
      for i:=1 to 3 do begin
        for j:=1 to 3 do begin
          r21[i,j]:=0.;
        end;
      end;
      r21[1,1]:=celr[1];
      r21[1,2]:=celr[2]*cce;
      r21[1,3]:=celr[3]*cbe;
      r21[2,2]:=celr[2]*sce;
      r21[2,3]:=-celr[3]*sbe*ca;
      r21[3,3]:=1./cel[3];

{-----Orthonormalisation du rseau direct R20}
      matinv(r21,r12);
      mat_mult(r21,r10,r20);
      mat_mult(r01,r12,r02);
      mat_mult(r01,r10,aux);

{-----Matrices R03 et R30 de dpart}
      for i:=1 to 3 do begin
       for j:=1 to 3 do begin
          r23[i,j]:=0;
          r32[i,j]:=0;
          if (i = j) then begin
            r23[i,j]:=1;
            r32[i,j]:=1;
          end;
       end;
      end;
      mat_mult(r02,r23,r03);
      {r23=r32!}
      mat_mult(r23,r20,r30);

  end;

procedure wr       (var nomf:TextFile; Titre:string; Mat:matrice3);{
-------------------------------------------------------------------}
var
  i,j  : integer;
begin
  writeln(nomf,Titre);
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
     write(nomf,mat[i,j]:12:6);
    end;
    writeln(nomf);
  end;
end;

procedure lit (var nomf:TextFile; var Mat:matrice3);{
----------------------------------------------------}
var
  i,j  : integer;
begin
  readln(nomf);
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
     read(nomf,mat[i,j]);
    end;
    readln(nomf);
  end;
end;

{Normalisation d'un vecteur dans l'espace orthonorm}
procedure norm(var p,q,r : real);{
---------------------------------}
var
  x : real;
begin
  x:=sqrt(p*p+q*q+r*r);
  p:=p/x;
  q:=q/x;
  r:=r/x;
end;

{Normalisation dans l'espace direct}
procedure normd(var x,y,z:real);{
--------------------------------}
var
  den : real;

begin
   den:=sqrt(x*x*a*a+y*y*b*b+z*z*c*c+2*(x*y*a*b*cc+y*z*b*c*ca+z*x*c*a*cb));
   x:=x/den;
   y:=y/den;
   z:=z/den;
end;

{Normalisation dans l'espace rciproque}
procedure normr(var x,y,z:real);{
--------------------------------}
var
  den : real;
begin
  den:=sqrt(x*x*ae*ae+y*y*be*be+z*z*ce*ce+
  2*(x*y*ae*be*cce+y*z*be*ce*cae+z*x*ce*ae*cbe));
  x:=x/den;
  y:=y/den;
  z:=z/den;
end;

{Calcul de la matrice connaissant seulement le icas(me) vecteur}
procedure rot(p0,q0,r0: real; icas:integer);{
--------------------------------------------}
var
  p,q,r,racine                            : real;
  i,j                                     : integer;

begin
   trfo(p0,q0,r0,p,q,r,r20);
   norm(p,q,r);

   for i:=1 to 3 do begin
     for j:=1 to 3 do begin
       r32[i,j]:=0;
     end;
   end;

{P,Q,R sur X }
    if(icas = 1)then begin
      racine:=sqrt(r*r+q*q);
      if (racine < 0.0001)then begin
        for i:=1 to 3 do begin
          for j:=1 to 3 do begin
            r32[i,j]:=0;
          end;
          r32[i,i]:=1;
        end;
      end;
      if (racine >= 0.0001)then begin
        r32[1,1]:=p;
        r32[1,2]:=q;
        r32[1,3]:=r;
        r32[2,1]:=0;
        r32[2,2]:=-r/racine;
        r32[2,3]:=q/racine;
        r32[3,1]:=racine;
        r32[3,2]:=-p*q/racine;
        r32[3,3]:=-p*r/racine;
      end;
    end;

{p,p,r sur Y}
    if(icas = 2)then begin
      racine:=sqrt(p*p+r*r);
      if(racine < 0.0001)then begin
        for i:=1 to 3 do begin
          for j:=1 to 3 do begin
            r32[i,j]:=0;
          end;
          r32[i,i]:=1;
        end;
      end;
      if(racine >= 0.0001) then begin
        r32[1,1]:=-p*q/racine;
        r32[1,2]:=racine;
        r32[1,3]:=-q*r/racine;
        r32[2,1]:=p;
        r32[2,2]:=q ;
        r32[2,3]:=r;
        r32[3,1]:=r/racine ;
        r32[3,2]:=0;
        r32[3,3]:=-p/racine;
      end;
    end;

{p,p,r sur Z }
    if(icas = 3)then begin
      racine:=sqrt(p*p+q*q);
      if( racine < 0.0001)then begin
        for i:=1 to 3 do begin
          for j:=1 to 3 do begin
            r32[i,j]:=0;
          end;
          r32[i,i]:=1;
        end;
      end;
      if( racine >= 0.0001)then begin
        r32[1,1]:=q/racine;
        r32[1,2]:=-p/racine;
        r32[1,3]:=0;
        r32[2,1]:=p*r/racine;
        r32[2,2]:=q*r/racine;
        r32[2,3]:=-racine;
        r32[3,1]:=p;
        r32[3,2]:=q;
        r32[3,3]:=r;
      end;
    end;

    matinv(r32,r23);
    mat_mult(r32,r21,r31);
    matinv(r31,r13);
end;

procedure erreur(s:string);{
---------------------------}
begin
   MessageDlg(s,mtWarning,[mbOK],0);
end;

procedure ecrit_fichier_usr;{
----------------------------}
var
  nomf                                  : TexTfile;
  i,j,k                                 : integer;

begin
  AssignFile(nomf,nom_fichier_usr);
  rewrite(nomf);

  writeln(nomf,nomstruc);

  {Maille}
  writeln(nomf,'CELL');
  for i:=1 to 6 do write(nomf,cel[i]:7:3);
  writeln(nomf);

  writeln(nomf,'SPGR');
  writeln(nomf,groupe_espace);

  {Eventuellement positions motif}
  writeln(nomf,'POSI');
  if(groupe_espace = '')then begin
    writeln(nomf,centre:3);
    writeln(nomf,nbtrans:3);
    if(nbtrans > 0)then begin
      for j:=1 to nbtrans do
      writeln(nomf,tx[j]:8:5,ty[j]:8:5,tz[j]:8:5);
    end;
    writeln(nomf,npos:3);
    for i:=1 to npos do begin
      for j:=1 to 3 do begin
        for k:=1 to 3 do write(nomf,tp[i,j,k]:4:0);
        write(nomf,tp[i,j,4]:10:6);
      end;
      writeln(nomf);
    end;
  end;

  {Coordonnes motif asymtrique}
  writeln(nomf,'COOR');
  writeln(nomf,nb_motif:3);
    for i:=1 to nb_motif do begin
     with motif[i] do begin
       writeln(nomf,nom:4,'     ',x:10:6,y:10:6,z:10:6);
       write(nomf,'         ');
       for j:=1 to 6 do write(nomf,beta[j]:10:6);
       writeln(nomf);
     end;
  end;

  CloseFile(nomf);
  erreur('Le fichier '+extractFileName(nom_fichier_usr)+' a t cr');

end;

procedure ecrit_fichier_opt;{
----------------------------}
var
  nomf                                  : TexTfile;
  i,j,k                                 : integer;

begin
  //nom_fichier_opt:=nomstruc+'.opt';
  nom_fichier_opt:='desmol.opt';
  AssignFile(nomf,nom_fichier_opt);
  rewrite(nomf);

  writeln(nomf,nomstruc);

  {Maille}
  writeln(nomf,'CELL');
  for i:=1 to 6 do write(nomf,cel[i]:8:3);
  writeln(nomf);

  {Positions}
  writeln(nomf,'POSI');
  writeln(nomf,npos_sel:3);
  for i:=1 to npos_sel do begin
    for j:=1 to 3 do begin
      for k:=1 to 3 do write(nomf,tp_sel[i,j,k]:4:0);
      write(nomf,tp_sel[i,j,4]:10:6);
    end;
    writeln(nomf);
  end;

  {Coordonnes }
  writeln(nomf,'COOR');
  writeln(nomf,nb_child:3);
  for i:=1 to nb_child do begin
    with child[i] do begin
       writeln(nomf,nom:4,'     ',x:10:6,y:10:6,z:10:6);
       write(nomf,'         ');
       for j:=1 to 6 do write(nomf,beta[j]:10:6);
       writeln(nomf);
    end;
  end;

  writeln(nomf,'LIAI');
  writeln(nomf,nb_bonds:4);
  if (nb_bonds > 0)then
  for i:=1 to nb_bonds do writeln(nomf,bonds[i].atome1:5,bonds[i].atome2:5);

  writeln(nomf,'MATR');
  wr(nomf,'R32',r32);
  wr(nomf,'R23',r23);
  CloseFile(nomf);
end;

 function uptrim(var s:string): string;
//----------------------------- -------
begin
  s:=trim(s);
  uptrim:=uppercase(s);
end;

//Recherche des sortes d'atomes (lments)
procedure set_typ_elements;
//-------------------------
var
  s2                     : string[2];
  s                      : string;
  i,j,ier                : integer;
  ray                    : real;
  nomfich                : TextFile;
begin
  s:=copy(motif[1].nom,1,2);
  element[1].nom:=uptrim(s);
  nb_typ_element:=1;
  for i:=2 to nb_motif do begin
    s:=copy(motif[i].nom,1,2);
    s2:=uptrim(s);
    ier:=0;
    for j:=1 to nb_typ_element do begin
      if (s2 = element[j].nom)then begin
        ier:=1;
        break;
      end;
    end;
    if (ier = 0)then begin
      nb_typ_element:=nb_typ_element+1;
      element[nb_typ_element].nom:=s2;
    end;
  end;

  //Lecture des rayons
  assignFile(nomfich,'rayons.cst');
  reset(nomfich);
  while not eof(nomfich) do begin
    readln(nomfich,s);
    readln(nomfich,ray);
    s2:=uptrim(s);
    for i:=1 to nb_typ_element do begin
      if (element[i].nom = s2)then begin
        element[i].rayon:=ray;
        break;
      end;
    end;
  end;
  closeFile(nomfich);

end;

{Etablissement des types de liaisons}
procedure set_typ_liaisons;{
----------------------------}
var
  i,j                   : integer;
  s2                    : string[2];
  s                     : string;
begin
  set_typ_elements;
  nb_typ_liai:=0;
  for i:=1 to nb_typ_element do begin
    for j:= i to nb_typ_element do begin
       nb_typ_liai:=nb_typ_liai+1;
       with tableau_liai[nb_typ_liai] do begin
         atome1:=trim(element[i].nom);
         atome2:=trim(element[j].nom);
         distance:=1.1*(element[i].rayon+element[j].rayon);
       end;
    end;
  end;

  //Set des no d'lment des atomes du motif
  if (nb_motif > 0) then begin
    for i:=1 to nb_motif do begin
      s:=copy(motif[i].nom,1,2);
      s2:=uptrim(s);
      for j:=1 to nb_typ_liai do begin
        if (element[j].nom = s2) then begin
          motif[i].no_element:=j;
          break;
        end;
      end;
    end;
  end;
end;

procedure lit_fichier_usr;
//------------------------
var
  nomf                        : TextFile;
  i,j,no,ier                  : integer;
  s                           : string;
  s4                          : string[4];
  tableau                     : tab_reel;
begin
     AssignFile(nomf,nom_fichier_usr);
     reset(nomf);

     {nom}
     readln(nomf,nomstruc);

     {CELL}
     readln(nomf);
     readln(nomf,s);
     dec(s,tableau,no,ier);
     for i:=1 to 6 do cel[i]:=tableau[i];

     //SPGR
     readln(nomf);
     readln(nomf,groupe_espace);
     groupe_espace:=trim(groupe_espace);

     //POSI
     if (groupe_espace = '')then begin
       readln(nomf);{'POSI'}
       readln(nomf,centre);
       readln(nomf,nbtrans);
       for i:=1 to nbtrans do readln(nomf,tx[i],ty[i],tz[i]);
       readln(nomf,npos);
       for i:=1 to npos do begin
         for j:=1 to 3 do read(nomf,tp[i,j,1],tp[i,j,2],tp[i,j,3],tp[i,j,4]);
         readln(nomf);
       end;
     end;

     {Coordonnes du motif asymtrique}
     //COOR
     readln(nomf);
     readln(nomf);
     nb_motif:=0;
     while not eof(nomf) do begin
       nb_motif:=nb_motif+1;
       with motif[nb_motif] do begin
         readln(nomf,s4,s);
         nom:=s4;
         dec(s,tableau,no,ier);
         x:=tableau[1];
         y:=tableau[2];
         z:=tableau[3];
         readln(nomf,s);
         dec(s,tableau,no,ier);
         if (no = 0)then begin
           erreur('erreur de lecture dans le fichier de donnes');
           closefile(nomf);
         exit;
       end;
         for i:=1 to 6 do beta[i]:=tableau[i];
         if (nom[1] = 'D') or (nom[1] = 'H') then begin
           nom[1]:='H';
           beta[1]:=1;
         end;
       end;
    end;
    set_typ_liaisons;
    {Fin de lecture}
    CloseFile(nomf);

end;

//Extrait jusqu'a 10 rels d'une chaine s. no:nb de rels trouvs(==>tableau )
procedure dec(s:string; var tableau:tab_reel; var no,ier:integer);
//----------------------------------------------------------------
var
  s1                            : string;
  i,j,l                         : integer;
 no_erreur                      : array[1..10] of integer;

begin
  s:=trim(s);
  l:=length(s);
  no:=0;
  for i:=1 to l do begin
    if (s[i] = ',') then s[i]:=' ';
  end;

  while (l > 0) do begin
    j:=pos(' ',s);
    if (j = 0)then j:=l;
    no:=no+1;
    s1:=copy(s,1,j);
    s1:=trim(s1);
    no_erreur[no]:=0;
    val(s1,tableau[no],ier);
    if(ier <> 0)then  no_erreur[no]:=1;
    delete(s,1,j);
    s:=trim(s);
    l:=length(s);
  end;
 end;

procedure decchaine(s: string; var line: vecteur3; var t: real);{
----------------------------------------------------------------
Dcode une sous chaine d'une chaine-position}
var
  i,j,ier,l,k          : integer;

const
  code_pos      : array[1..10] of string =
  ('X-Y','Y-X','-Y+X','-X+Y','-X','-Y', '-Z','X','Y','Z');
  ligne         : array[1..30] of real   =
  (1,-1,0,-1,1,0,1,-1,0,-1,1,0,-1,0,0,0,-1,0,0,0,-1,1,0,0,0,1,0,0,0,1);

begin

   {suppression des formats fortran}
   i:=pos(' .',s);
   if(i > 0)then s[i]:='0';
   i:=pos('. ',s);
   if(i > 0)then s[i+1]:='0';

   {recherche vecteur de position }
   for i:=1 to 10 do begin
     j:=pos(code_pos[i],s);
     if (j > 0) then begin
       k:=(i-1)*3 ;
       line[1]:=ligne[k+1];
       line[2]:=ligne[k+2];
       line[3]:=ligne[k+3];
       l:=length(code_pos[i]);
       delete(s,j,l);
       break;
     end;
   end;

   //Translations
   s:=trim(s);
   val(s,t,ier);
end;

procedure decsym(s: string; var r:matrice3; var t:vecteur3); {
-------------------------------------------------------------}
  {Dcodage d'une position, Sparation des 3 sous-chaines, par recherche des
  virgules}
var
  s1,s2,s3                   : string;
  i                          : integer;
  v                          : vecteur3;
begin

  i:=pos(',',s);
  s1:=copy(s,1,i);
  s1[i]:=' ';
  delete(s,1,i);
  s:=trim(s);

  i:=pos(',',s);
  s2:=copy(s,1,i);
  s2[i]:=' ';
  delete(s,1,i);
  s:=trim(s);

  s3:=s;

  s1:=trim(s1);
  s2:=trim(s2);
  s3:=trim(s);

  {Dcodage de chaque sous-chaine}
  for i:=1 to 3 do begin
    v[i]:=0;
    t[i]:=0;
  end;
  decchaine(s1,v,t[1]);
  for i:=1 to 3 do r[1,i]:=v[i];
  decchaine(s2,v,t[2]);
  for i:=1 to 3 do r[2,i]:=v[i];
  decchaine(s3,v,t[3]);
  for i:=1 to 3 do r[3,i]:=v[i];
end;

Procedure traite_nom_element(var s:string);
//----------------------------------------
var
  c          : char;
begin
  c:=s[2];
  if (ord(c) > 47) and (ord(c) < 58)then insert(' ',s,2);
end;

{Lecture des cartes du fichier de sortie de Shelx76 ou 93}
procedure licarte;{
------------------}
var
  nomf                                          : TextFile;
  s5                                            : string[5];
  s,s1                                          : string;
  depi2                                         : real;
  u                                             : array[1..6] of real;
  valeur                                        : tab_reel;
  nb_valeurs,ier,i,j,no                         : integer;
  r                                             : matrice3;
  t                                             : vecteur3;

begin
  AssignFile(nomf,nom_fichier_shelx);
  reset(nomf);

  {Dtection des cartes de symtrie}
  npos:=1;
  for i:=1 to 3 do begin
    for j:=1 to 4 do begin
      tp[1,i,j]:=0;
      if(i = j)then tp[1,i,j]:=1;
    end;
  end;

  npos:=1;
  //On rajoute la position triviale
  for i:=1 to 4 do
  for j:=1 to 4 do begin
      tp[1,i,j]:=0;
      if (i = j)then tp[1,i,j]:=1;
 end;

  while not eof(nomf) do begin
    readln(nomf,s5,s);
    if(s5 = ' SYMM')then begin
      decsym(s,r,t);
      npos:=npos+1;
      for i:=1 to 3 do begin
        for j:= 1 to 3 do tp[npos,i,j]:=r[i,j];
        tp[npos,i,4]:=t[i];
      end;
    end;
    if(s5 = '')then break;
  end;

  closefile(nomf);
  AssignFile(nomf,nomstruc+'.out');
  reset(nomf);

  {Dtection de la maille}
  while not eof(nomf) do begin
    readln(nomf,s5,s);
    if(s5 = ' CELL') then  begin
      dec(s,valeur,no,ier);
      for i:=1 to 6 do cel[i]:=valeur[i+1];
      break;
    end;
  end;

  {Dcodage du mode de rseau}
  closefile(nomf);
  AssignFile(nomf,nomstruc+'.out');
  reset(nomf);
  ibrav:=0;
  while not eof(nomf) do begin
    readln(nomf,s5,s);
    if (s5 = ' LATT')then begin
      s:=trim(s);
      val(s,ibrav,ier);
      if(ibrav > 0)then centre:=1;
      if(ibrav < 0)then begin
        centre:=-1;
        ibrav:=-ibrav
      end;
      break;
    end;
  end;
  if (ibrav = 0)then ibrav:=1;

  {Etablissemnt des translations}
  case ibrav of
       1 : nbtrans:=0;
       2 : begin
             nbtrans:=1;
             tx[1]:=0.5;
             ty[1]:=0.5;
             tz[1]:=0.5;
           end;
       4 : begin
             nbtrans:=3;
             tx[1]:=0.5;
             ty[1]:=0.5;
             tz[1]:=0;
             tx[2]:=0.5;
             ty[2]:=0;
             tz[2]:=0.5;
             tx[3]:=0;
             ty[3]:=0.5;
             tz[3]:=0.5;
           end;
       5 : begin
             nbtrans:=1;
             tx[1]:=0;
             ty[1]:=0.5;
             tz[1]:=0.5;
           end;
       6 : begin
             nbtrans:=1;
             tx[1]:=0.5;
             ty[1]:=0;
             tz[1]:=0.5;
           end;
       7 : begin
             nbtrans:=1;
             tx[1]:=0.5;
             ty[1]:=0.5;
             tz[1]:=0;
           end;

  end;

  {Lecture des coordonnes du motif asymtrique}
  setmat;

  {c--Pour le passage des uij aux beta!}
  depi2:=2.*3.14159*3.14159;
  u[1]:=depi2*celr[1]*celr[1];
  u[2]:=depi2*celr[2]*celr[2];
  u[3]:=depi2*celr[3]*celr[3];
  u[4]:=depi2*celr[1]*celr[2];
  u[5]:=depi2*celr[1]*celr[3];
  u[6]:=depi2*celr[2]*celr[3];

  //Lecture des coordonnes
  closefile(nomf);
  AssignFile(nomf,nomstruc+'.out');
  reset(nomf);
  nb_motif:=0;
  ier:=1;
  while not eof(nomf) do begin
    s:='';
    readln(nomf,s);
    if(type_fichier_shelx = 93)then s1:='ATOM           x         y         z';
    if(type_fichier_shelx = 76)then s1:='ATOM      X/A';
    if (pos(s1,s) > 0 ) then begin
      ier:=0;
      break;
    end;
  end;

  if (ier = 1)then begin
    erreur
    ('Pas d''atomes dtcts dans le fichier '+extractFileName(nom_fichier_shelx));
    closefile(nomf);
    exit;
  end;

  readln(nomf);
  while not eof(nomf) do begin
    readln(nomf,s5,s);
    if(s5 = '') then break;                                   //atome,x,y,etrc..
    readln(nomf);                                            //sigmas
    if(type_fichier_shelx = 93) then readln(nomf);           //ligne blanche
    for i:=1 to 10 do valeur[i]:=0;
    dec(s,valeur,nb_valeurs,ier);
    if(ier = 0) then begin
      nb_motif:=nb_motif+1;
      with motif[nb_motif] do begin
        s:=copy(s5,2,5);
        traite_nom_element(s);
        nom:=s;
        x:=valeur[1];
        y:=valeur[2];
        z:=valeur[3];
        beta[1]:=valeur[5]*u[1];
        beta[2]:=valeur[6]*u[2];
        beta[3]:=valeur[7]*u[3];
        beta[4]:=valeur[10]*u[4];
        beta[5]:=valeur[9]*u[5];
        beta[6]:=valeur[8]*u[6];
      end;
    end;
  end;
  closeFile(nomf);
  set_typ_liaisons;
end;


end.
